#![warn(clippy::all)]

mod commands;
mod personality;
mod queue;

use commands::*;

use std::{env, sync::Arc};

use anyhow::Result;
use parking_lot::Mutex;
use poise::serenity_prelude::{self as serenity};
use songbird::serenity::SerenityInit;

pub struct Data {
    queue: Arc<Mutex<queue::Queue>>,
}
pub type Error = Box<dyn std::error::Error + Send + Sync>;
pub type CommandContext<'a> = poise::Context<'a, Data, Error>;

async fn event_event_handler(
    _ctx: &serenity::Context,
    event: &poise::Event<'_>,
    _framework: poise::FrameworkContext<'_, Data, Error>,
    _user_data: &Data,
) -> Result<(), Error> {
    if let poise::Event::Ready { data_about_bot } = event {
        println!("{} is connected!", data_about_bot.user.name)
    }

    Ok(())
}

/// Registers slash commands in this guild or globally
#[poise::command(prefix_command, hide_in_help)]
async fn register(ctx: CommandContext<'_>) -> Result<(), Error> {
    poise::builtins::register_application_commands_buttons(ctx).await?;

    Ok(())
}

#[tokio::main]
async fn main() -> Result<()> {
    tracing_subscriber::fmt().init();

    let token =
        env::var("DISCORD_TOKEN").expect("Expected a bot token in the environment: DISCORD_TOKEN");

    let options = poise::FrameworkOptions {
        commands: vec![
            register(),
            join(),
            leave(),
            play(),
            queue(),
            stop(),
            skip(),
            pause(),
            resume(),
            clear(),
        ],
        event_handler: |ctx, event, framework, user_data| {
            Box::pin(event_event_handler(ctx, event, framework, user_data))
        },
        ..Default::default()
    };

    let intents = serenity::GatewayIntents::non_privileged();

    poise::Framework::builder()
        .token(token)
        .options(options)
        .intents(intents)
        .setup(|_ctx, _data, _framework| {
            Box::pin(async move {
                Ok(Data {
                    queue: Arc::new(Mutex::new(queue::Queue::new())),
                })
            })
        })
        .client_settings(|client_builder| client_builder.register_songbird())
        .run()
        .await?;

    Ok(())
}
